@@ -1,13 +1,13 @@
 from aiohttp import web
 
-from joule.models import Supervisor
+from joule.models.supervisor import Supervisor
 
 
 async def index(request):
     supervisor: Supervisor = request.app["supervisor"]
     resp = []
-    if 'statistics' in request.query:
-        get_stats=True
+    if 'statistics' in request.query and request.query['statistics'] != '0':
+        get_stats = True
     else:
         get_stats = False
 
@@ -16,7 +16,7 @@ async def index(request):
             "id": worker.uuid,
             "name": worker.name,
             "description": worker.description,
-            "has_interface": worker.has_interface,
+            "is_app": worker.is_app,
             "inputs": {},
             "outputs": {}}
         if get_stats:
@@ -35,16 +35,19 @@ async def info(request):
     supervisor: Supervisor = request.app["supervisor"]
     if 'name' in request.query:
         worker = [w for w in supervisor.workers if w.name == request.query['name']]
+    elif 'id' in request.query:
+        worker = [w for w in supervisor.workers if w.uuid == int(request.query['id'])]
     else:
-        return web.Response(text="specify a name", status=400)
+        return web.Response(text="specify a name or id", status=400)
     if len(worker) == 0:
         return web.Response(text="module does not exist", status=404)
     worker = worker[0]
+
     data = {
         "id": worker.uuid,
         "name": worker.name,
         "description": worker.description,
-        "has_interface": worker.has_interface,
+        "is_app": worker.is_app,
         "inputs": {},
         "outputs": {},
         "statistics": (await worker.statistics()).to_json()}
@@ -59,6 +62,8 @@ async def logs(request):
     supervisor: Supervisor = request.app["supervisor"]
     if 'name' in request.query:
         worker = [w for w in supervisor.workers if w.name == request.query['name']]
+    elif 'id' in request.query:
+        worker = [w for w in supervisor.workers if w.uuid == int(request.query['id'])]
     else:
         return web.Response(text="specify a name", status=400)
     if len(worker) == 0:
