@@ -24,7 +24,7 @@ class LocalPipe(Pipe):
     """
 
     def __init__(self, layout: str, loop: Loop = None, name: str = None,
-                 close_cb = None, debug: bool = False, stream=None):
+                 close_cb = None, debug: bool = False, stream=None, write_limit=0):
 
         super().__init__(name=name, layout=layout, stream=stream)
         if loop is None:
@@ -36,8 +36,9 @@ class LocalPipe(Pipe):
         self.closed = False
         self.read_buffer = np.empty((0,), dtype=self.dtype)
         self.close_cb = close_cb
+        self._reread = False
         # initialize buffer and queue
-        self.queue = asyncio.Queue(loop=loop)
+        self.queue = asyncio.Queue(loop=loop, maxsize=write_limit)
         self.queued_rows = 0
         self.last_index = 0
         self.direction = Pipe.DIRECTION.TWOWAY
@@ -47,7 +48,18 @@ class LocalPipe(Pipe):
         self._cache = None
 
     async def read(self, flatten=False):
+        if self._failed:
+            await self.close()
+            raise PipeError('pipe failed')
         self.interval_break = False
+
+        # if reread is set just return the old data
+        if self._reread:
+            self._reread = False
+            if len(self.read_buffer) == 0:
+                raise PipeError("No data left to reread")
+            return self._format_data(self.read_buffer, flatten)
+
         # if the queue is empty and we have old data, just return the old data
         if self.queue.empty() and len(self.read_buffer) > 0:
             return self._format_data(self.read_buffer, flatten)
@@ -75,9 +87,20 @@ class LocalPipe(Pipe):
         [1, 2, 3]
 
         """
+        if self._failed:
+            raise PipeError('pipe failed')
+
+        # if reread is set just return the old data
+        if self._reread:
+            self._reread = False
+            if len(self.read_buffer) == 0:
+                raise PipeError("No data left to reread")
+            return self._format_data(self.read_buffer, flatten)
+
         # if the queue is empty and we have old data, just return the old data
         if self.queue.empty() and len(self.read_buffer) > 0:
             return self._format_data(self.read_buffer, flatten)
+
         # if the buffer is empty and the queue is empty and the pipe is closed
         if self.queue.empty() and len(self.read_buffer) == 0 and self.closed:
             raise EmptyPipe()
@@ -111,6 +134,11 @@ class LocalPipe(Pipe):
             print("[%s:read] returning %d rows" % (self.name, len(self.read_buffer)))
         return self._format_data(self.read_buffer, flatten)
 
+    def reread_last(self):
+        if len(self.read_buffer) == 0:
+            raise PipeError("No data left to reread")
+        self._reread = True
+
     @property
     def end_of_interval(self):
         return self.interval_break
@@ -128,6 +156,11 @@ class LocalPipe(Pipe):
         self.read_buffer = self.read_buffer[num_rows:]
 
     async def write(self, data: np.ndarray):
+        if self._failed:
+            await self.close()
+            raise PipeError('pipe failed')
+        if self.closed:
+            raise PipeError("Cannot write to a closed pipe")
         if not self._validate_data(data):
             return
         # convert into a structured array
@@ -143,17 +176,26 @@ class LocalPipe(Pipe):
             await self._write(sarray)
 
     async def _write(self, sarray):
-
         # send data to subscribers
         for pipe in self.subscribers:
             await pipe.write(sarray)
 
-        self.queue.put_nowait(sarray)
+        # if the queue size is infinite do not wait
+        if self.queue.maxsize <= 0:
+            self.queue.put_nowait(sarray)
+        else:
+            # wait until a slot is available
+            await self.queue.put(sarray)
         self.queued_rows += len(sarray)
+        await asyncio.sleep(0)
         if self.debug:
             print("[%s:write] queueing block with [%d] rows" % (self.name, len(sarray)))
 
     def write_nowait(self, data):
+        if self._failed:
+            raise PipeError('pipe failed')
+        if self.closed:
+            raise PipeError("Cannot write to a closed pipe")
         if not self._validate_data(data):
             return
 
@@ -178,12 +220,18 @@ class LocalPipe(Pipe):
         self._cache_index = 0
 
     async def flush_cache(self):
+        if self.closed:
+            raise PipeError("Cannot write to a closed pipe")
         if self._cache_index > 0:
             await self._write(self._cache[:self._cache_index])
             self._cache_index = 0
             self._cache = np.empty(len(self._cache), self.dtype)
 
     async def close_interval(self):
+        if self._failed:
+            raise PipeError('pipe failed')
+        if self.closed:
+            raise PipeError("Cannot write to a closed pipe")
         if self.debug:
             print("[%s:write] closing interval" % self.name)
         if self._caching:
@@ -196,17 +244,33 @@ class LocalPipe(Pipe):
         unit testing
 
         """
+        if self._failed:
+            raise PipeError('pipe failed')
         if self.debug:
             print("[%s:write] closing interval" % self.name)
         self.queue.put_nowait(None)
 
+    def change_layout(self, layout: str):
+        self._layout = layout
+        self.read_buffer = np.empty((0,), dtype=self.dtype)
+        self.queued_rows = 0
+        self.last_index = 0
+        # caching
+        if self._caching:
+            self._cache = np.empty(len(self._cache), self.dtype)
+            self._cache_index = 0
+
     async def close(self):
-        self.closed = True
-        if self.close_cb is not None:
-            await self.close_cb()
+        if self.closed:
+            return
+        if self._caching:
+            await self.flush_cache()
         # close any subscribers
         for pipe in self.subscribers:
             await pipe.close()
+        self.closed = True
+        if self.close_cb is not None:
+            await self.close_cb()
 
     def close_nowait(self):
         """
@@ -214,8 +278,9 @@ class LocalPipe(Pipe):
         unit testing
 
         """
-        self.closed = True
         if len(self.subscribers) > 0:
             raise PipeError("cannot close_nowait subscribers, use async")
         if self.close_cb is not None:
             raise PipeError("close_cb cannot be executed, use async")
+        self.closed = True
+
